/*
 * CacheScreen.java
 * 
 * Copyright (c) David Pollack 2009-2010
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package ScrobbleBerry;

import net.rim.device.api.ui.*;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.*;

class CacheScreen extends MainScreen {
    ObjectListField list = new ObjectListField(ObjectListField.ELLIPSIS);
    ScrobblerSession _session = null;
    
    CacheScreen(ScrobblerSession session) {
        _session = session;
        
        LabelField _title = new LabelField(
                "ScrobbleBerry - Cache Manager",
                LabelField.ELLIPSIS | LabelField.USE_ALL_WIDTH);
        setTitle(_title);
        
        updateList();
        
        add(list);
        
        MenuItem _removeItem = new MenuItem("Remove Track", 1, 1) {
            public void run() {
                int selectedIndex = list.getSelectedIndex();
                if (selectedIndex >= 0) {
                    int response = Dialog.ask(Dialog.D_YES_NO,
                        "Are you sure you wish to remove \"" +
                        list.get(list, selectedIndex) +
                        "\" from the cache? This action is irreversible.",
                        Dialog.YES);
                    if (response == Dialog.YES) {
                        _session.removeScrobbleCacheAt(selectedIndex);
                        _session.updateStore();
                        updateList();
                    }
                } else {
                    Dialog.alert("The cache is currently empty.");
                }
            }
        };
        MenuItem _submitItem = new MenuItem("Submit Cache", 1, 1) {
            public void run() {
                if (_session.isConnected()) {
                    Status.show("Submitting...");
                    _session.submitCache();
                    _session.updateStore();
                    Dialog.alert("Cache submitted!");
                    updateList();
                } else {
                    Dialog.alert("You are not currently connected to Last.fm. " +
                        "Please connect from the main screen and try " +
                        "submitting the cache again.");
                }
            }
        };
        MenuItem _clearItem = new MenuItem("Clear Cache", 1, 1) {
            public void run() {
                int response = Dialog.ask(Dialog.D_YES_NO,
                    "Warning: This will permanently delete all " +
                    _session.cacheSize() +
                    " scrobbles that have not yet been submitted to Last.fm. " +
                    "Are you sure you wish to clear your cache?",
                    Dialog.NO);
                if (response == Dialog.YES) {
                    _session.clearCache();
                    _session.updateStore();
                    Dialog.alert("Cache cleared!");
                    updateList();
                }
            }
        };
        
        addMenuItem(_removeItem);
        addMenuItem(_submitItem);
        addMenuItem(_clearItem);
    }
    
    void updateList() {
        int size = _session.cacheSize();
        String[] scrobbleList = new String[size];
        for (int i = 0; i < size; i++) {
            Scrobble currScrobble = _session.getScrobbleCacheAt(i);
            scrobbleList[i] = currScrobble.getArtist() +
                " - " +
                currScrobble.getTrack();
        }
        list.set(scrobbleList);
    }
}
